// TailDataInterpreter.cpp : Implementation of CTailDataInterpreter

#include "stdafx.h"
#include "TailDataInterpreter.h"
#include <algorithm>

// CTailDataInterpreter

const HRESULT DAIN_S_EOF = MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_ITF, 0x0401);

const int MAX_LINES = 100;

const char * pszEOL = "\r\n";
const int EOL_SIZE = strlen(pszEOL);

HRESULT CTailDataInterpreter::convertDataToLines(DATALIST * pmem)
{
	if (pmem->empty())
		return S_OK;


	// Reverse iterate from the end of the buffer to front looking for the first 100
	// lines (EOL == CRLF)

	BYTE * pLastPos = &(pmem->back());
	BYTE * pFirstPos = &(pmem->front());

	for (BYTE * pFind = pLastPos; (pFind >= pFirstPos) && (m_lstLines.size() < MAX_LINES); --pFind)
	{
		if (memcmp(pFind, pszEOL, EOL_SIZE) == 0)
		{
			CString strNewLine;

			int nLen = (int)(pLastPos - pFind - EOL_SIZE);
			if (nLen > 0)
			{
				// Assume the string of bytes is UTF-8 and convert to a wide string

				BYTE * pLine = pFind + EOL_SIZE;
				int nWideSize = ::MultiByteToWideChar(CP_UTF8, 0, (const char *)pLine, nLen, NULL, 0);
				if (nWideSize > 0)
				{
					wchar_t* pszBuff = strNewLine.GetBufferSetLength(nWideSize);
					nWideSize = MultiByteToWideChar(CP_UTF8, 0, (const char *)pLine, nLen, pszBuff, nWideSize);
					strNewLine.ReleaseBuffer(nWideSize);
				}
			}
			m_lstLines.push_front(strNewLine);

			pLastPos = pFind;
		}
	}
	
	return S_OK;
}


HRESULT CTailDataInterpreter::Open(BSTR bstrPathName)
{
	// Open the file and read the last 100k from the file

	HRESULT hr = S_OK;

	m_lstLines.clear();

	CComPtr<IStream> ptrFile;
	hr = SHCreateStreamOnFileEx(bstrPathName, STGM_READ | STGM_SHARE_DENY_NONE, FILE_ATTRIBUTE_NORMAL, FALSE, NULL, &ptrFile);
	if (SUCCEEDED(hr))
	{
		STATSTG stat = { 0 };
		hr = ptrFile->Stat(&stat, NULL);
		if (SUCCEEDED(hr))
		{
			// Set the size of the buffer up to 100k

			UINT64 nSize = std::min<UINT64>(stat.cbSize.QuadPart, 100 * 1024);

			if (nSize > 0)
			{
				DATALIST mem;
				mem.resize(nSize + EOL_SIZE);

				// Seek to the last 100k of the file (if the file is larger than 100k)

				if (nSize < stat.cbSize.QuadPart)
				{
					LARGE_INTEGER nSeekPos = { 0 };
					nSeekPos.QuadPart = stat.cbSize.QuadPart - nSize;

					hr = ptrFile->Seek(nSeekPos, STREAM_SEEK_SET, NULL);
				}

				if (SUCCEEDED(hr))
				{
					// Read the data in and then convert that data into lines

					ULONG nRead = 0;
					memcpy(&mem[0], pszEOL, EOL_SIZE);	// prefix the buffer with EOL just to make processing easier.

					hr = ptrFile->Read(&mem[EOL_SIZE], (ULONG)nSize, &nRead);
					if (SUCCEEDED(hr))
						hr = convertDataToLines(&mem);
				}
			}
		}
	}

	m_itNextLine = m_lstLines.begin();
	m_nLineNo = 1;

	return hr;
}

HRESULT CTailDataInterpreter::Close()
{
	return S_OK;
}

HRESULT CTailDataInterpreter::GetNextLine(BSTR * pbstrText, LONG * pnLine)
{
	// Keep reading lines from the buffer until we reach the end

	if (m_itNextLine != m_lstLines.end())
	{
		CComBSTR bstr(*m_itNextLine);
		*pbstrText = bstr.Detach();
		*pnLine = m_nLineNo;

		++m_itNextLine;
		++m_nLineNo;

		return S_OK;
	}
	else
	{
		// No more lines send DAIN_S_EOF

		return DAIN_S_EOF;
	}
}