// TailDataInterpreter.h : Declaration of the CTailDataInterpreter

// Description: This is a sample data interpreter to return just the last 100 lines of a file or the last 100K, whichever is first.
//				The premise is that the user may be searching large log files and is only interested in the last 100 lines of the log file.
//				Assumes the file is in UTF-8 format.

#pragma once
#include "resource.h"       // main symbols



#include "SamplePlugIn_i.h"
#include <vector>
#include <list>
#include <atlstr.h>


using namespace ATL;


// CTailDataInterpreter


class ATL_NO_VTABLE CTailDataInterpreter :
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CTailDataInterpreter, &CLSID_TailDataInterpreter>,
	public ITailDataInterpreter,
	public IExtDataInterpreter
{
public:
	CTailDataInterpreter()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_TAILDATAINTERPRETER)

DECLARE_NOT_AGGREGATABLE(CTailDataInterpreter)

BEGIN_COM_MAP(CTailDataInterpreter)
	COM_INTERFACE_ENTRY(ITailDataInterpreter)
	COM_INTERFACE_ENTRY(IExtDataInterpreter)
END_COM_MAP()



	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

public:

	STDMETHOD(Open)(BSTR bstrPathName);

	STDMETHOD(Close)();

	STDMETHOD(GetNextLine)(BSTR * pbstrText, LONG * pnLine);

private:
	typedef std::vector<BYTE> DATALIST;
	typedef std::list<CString> STRINGLIST;

	STRINGLIST m_lstLines;
	STRINGLIST::const_iterator m_itNextLine;
	int m_nLineNo;


	HRESULT convertDataToLines(DATALIST * pmem);


};

OBJECT_ENTRY_AUTO(__uuidof(TailDataInterpreter), CTailDataInterpreter)
